@@ -410,7 +410,7 @@ function MOI.Utilities.pass_nonvariable_constraints(
     constraint_types,
 )
     if Variable.has_bridges(Variable.bridges(dest))
-        # The functions may contained bridged variables which needs to be
+        # The functions may contain bridged variables which needs to be
         # substituted so we use the fallback.
         MOI.Utilities.pass_nonvariable_constraints_fallback(
             dest,
@@ -1749,6 +1749,9 @@ function MOI.set(
     return throw(MOI.SettingVariableIndexNotAllowed())
 end
 
+_f_type(::Type{S}) where {S<:MOI.AbstractScalarSet} = MOI.VariableIndex
+_f_type(::Type{S}) where {S<:MOI.AbstractVectorSet} = MOI.VectorOfVariables
+
 function MOI.get(
     model::MOI.ModelLike,
     attr::MOI.ConstraintConflictStatus,
@@ -1765,6 +1768,17 @@ function MOI.get(
             end
         end
     end
+    for (S,) in MOI.Bridges.added_constrained_variable_types(typeof(bridge))
+        F = _f_type(S)
+        for ci in MOI.get(bridge, MOI.ListOfConstraintIndices{F,S}())
+            status = MOI.get(model, attr, ci)
+            if status == MOI.IN_CONFLICT
+                return status
+            elseif status == MOI.MAYBE_IN_CONFLICT
+                ret = status
+            end
+        end
+    end
     return ret
 end
 
